/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is mozilla.org view-source front-end.
 *
 * The Initial Developer of the Original Code is mozilla.org.
 * Portions created by the Initial Developer are Copyright (C) 2002
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Jeroen Groenenboom <jagrboom@zonnet.nl> (Original Author)
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

var bMHPrefsWindow = null;

function BMHPrefsWindow() {
    window.title = "BookmarksHome Options";
    this.engineMenu = document.getElementById( "engine" );
    // changed hyphens in id to underscores because of bug
    this.tMCPField = document.getElementById( "table_main_cellpadding" );
    this.tCCSField = document.getElementById( "table_column_cellspacing" );
    this.nrOfColumnsField = document.getElementById( "nrOfColumns" );
    this.maxNameLengthField = document.getElementById( "maxNameLength" );
    this.openFolderInGroup = document.getElementById( "openFolderIn" );
    this.stylesheetField = document.getElementById( "stylesheet" );
    this.excludeMenu = document.getElementById( "excludeIndex" );
    this.inOrExcludedFoldersBox = document.getElementById( "inOrExcludedFolders" );
    this.subfolderCheckbox = document.getElementById( "subfolderCheckbox" );
    this.folderCheckboxArray = new Array();
    // minimum values for integer text fields
    this.minValues = new Array();
    this.minValues["table_main_cellpadding"] = 0;
    this.minValues["table_column_cellspacing"] = 0;
    this.minValues["nrOfColumns"] = 1;
    this.minValues["maxNameLength"] = 1;
}


BMHPrefsWindow.prototype = {

init: function() {
	  var popup = document.getElementById( "enginePopup" );
	  var item;
	  for( var i = 0; i < bMHPrefs.engines.length; i++ ) {
	      item = document.createElement( "menuitem" );
	      item.setAttribute( "label", bMHPrefs.engines[i].name );
	      popup.appendChild( item );
	  }
	  this.engineMenu.selectedIndex = bMHPrefs.getPref( "engine" );
	  this.tMCPField.value = bMHPrefs.getPref( "table_main_cellpadding" ); 
	  this.tCCSField.value = bMHPrefs.getPref( "table_column_cellspacing" );
	  this.nrOfColumnsField.value = bMHPrefs.getPref( "nrOfColumns" );
	  this.maxNameLengthField.value = bMHPrefs.getPref( "maxNameLength" );
	  this.openFolderInGroup.selectedIndex = bMHPrefs.getPref( "openFolderIn" );
	  this.stylesheetField.value = bMHPrefs.getPref( "stylesheet" );
	  this.excludeMenu.selectedIndex = bMHPrefs.getPref( "excludeIndex" );
	  this.createFolders();
      },

reset: function() {
	   var selectedPanel = document.getElementById( "prefsPanels" ).selectedIndex;
	   switch( selectedPanel ) {
	       case 0:
		   this.engineMenu.selectedIndex = bMHPrefs.getDefault( "engine" ).value;
		   this.tMCPField.value = bMHPrefs.getDefault( "table_main_cellpadding" ).value; 
		   this.tCCSField.value = bMHPrefs.getDefault( "table_column_cellspacing" ).value;
		   this.nrOfColumnsField.value = bMHPrefs.getDefault( "nrOfColumns" ).value;
		   this.maxNameLengthField.value = bMHPrefs.getDefault( "maxNameLength" ).value;
		   this.openFolderInGroup.selectedIndex = bMHPrefs.getDefault( "openFolderIn" ).value;
		   break;
	       case 1:
		   this.stylesheetField.value = bMHPrefs.getDefault( "stylesheet" ).value;
		   break;
	       case 2:
		   this.excludeMenu.selectedIndex = bMHPrefs.getDefault( "excludeIndex" ).value;
		   for( var i = 0; i < this.folderCheckboxArray.length; i++ ) {
		       this.folderCheckboxArray[i].node.checked = false;
		   }
		   break;
	   }
       },

close: function() {
	   window.close();
       },

apply: function() {
	   var prefs = new Array( bMHPrefs.getNrOfPrefs() );
	   var prefIndex = -1;
	   var sortedInOrExcludedFolders, inOrExcludedString, i;
	   try {
	       prefs[++prefIndex] = this.parseValue( this.engineMenu.id, this.engineMenu.selectedIndex );
	       prefs[++prefIndex] = this.parseValue( this.tMCPField.id, this.tMCPField.value );
	       prefs[++prefIndex] = this.parseValue( this.tCCSField.id, this.tCCSField.value );
	       prefs[++prefIndex] = this.parseValue( this.nrOfColumnsField.id, this.nrOfColumnsField.value );
	       prefs[++prefIndex] = this.parseValue( this.maxNameLengthField.id, this.maxNameLengthField.value );
	       prefs[++prefIndex] = this.parseValue( this.openFolderInGroup.id, this.openFolderInGroup.selectedIndex );
	       prefs[++prefIndex] = this.parseValue( this.stylesheetField.id, this.stylesheetField.value );
	       prefs[++prefIndex] = this.parseValue( this.excludeMenu.id, this.excludeMenu.selectedIndex );
	       sortedInOrExcludedFolders = new Array();
	       for( i = 0; i < this.folderCheckboxArray.length; i++ ) {
		   if( this.folderCheckboxArray[i].node.checked ) {
		       sortedInOrExcludedFolders.push( this.folderCheckboxArray[i].value.toString() );
		   }
	       }
	       // sort excluded folders for binary search
	       sortedInOrExcludedFolders.sort();
	       inOrExcludedString = "";
	       for ( i = 0; i < sortedInOrExcludedFolders.length; i++ ) {
		   inOrExcludedString += ( "|" + sortedInOrExcludedFolders[i] );
	       }
	       if( inOrExcludedString.length > 0 ) {
		   // remove first bar
		   inOrExcludedString = inOrExcludedString.substr( 1 );
	       }
	       prefs[++prefIndex] = this.parseValue( this.inOrExcludedFoldersBox.id, inOrExcludedString );
	       // set prefs
	       for( prefIndex = 0; prefIndex < prefs.length; prefIndex++ ) {
		   bMHPrefs.setPref( prefs[prefIndex].name, prefs[prefIndex].type, prefs[prefIndex].value );
	       }
	       // apply prefs
	       window.opener.location.reload();
	   }
	   catch( exception ) {
	       // inproper value in field
	       alert( exception );
	   }
       },

parseValue: function( name, value) {
		var type = bMHPrefs.getDefault( name ).type;
		var minVal = this.minValues[name];
		if( minVal == null ) {
		    // value does not need to be checked
		    return { name: name, type: type, value: value };
		}
		value = parseInt( value );
		if( isNaN( value ) || value < minVal ) {
		    throw name + " does not have a proper value";
		}
		return { name: name, type: type, value: value };
	    },

checkSubfolders: function( node ) {
		     if( this.subfolderCheckbox.checked ) {
			 // id is "fcb" + index
			 var startIndex = parseInt( node.id.substr( 3 ));
			 var regExp = new RegExp( "^" + this.folderCheckboxArray[startIndex].value );
			 var i = startIndex + 1;
			 while( i < this.folderCheckboxArray.length &&
				 regExp.test( this.folderCheckboxArray[i].value )) {
			     this.folderCheckboxArray[i].node.checked = 
				 this.folderCheckboxArray[startIndex].node.checked;
			     i++;
			 }
		     }
		 },

createFolders: function() {
		   // RDF variables
		   var NC_NS = "http://home.netscape.com/NC-rdf#";

		   var kRDFContractID = "@mozilla.org/rdf/rdf-service;1";
		   var kRDFSVCIID = Components.interfaces.nsIRDFService;
		   var kRDFRSCIID = Components.interfaces.nsIRDFResource;
		   var kRDFLITIID = Components.interfaces.nsIRDFLiteral;
		   var RDF = Components.classes[kRDFContractID].getService( kRDFSVCIID );

		   var kRDFCContractID = "@mozilla.org/rdf/container;1";
		   var kRDFCIID = Components.interfaces.nsIRDFContainer;
		   var RDFC = Components.classes[kRDFCContractID].createInstance( kRDFCIID );
		   var kRDFCUContractID = "@mozilla.org/rdf/container-utils;1";
		   var kRDFCUIID = Components.interfaces.nsIRDFContainerUtils;
		   var RDFCU = Components.classes[kRDFCUContractID].getService(kRDFCUIID);

		   var kBMSVCIID = Components.interfaces.nsIBookmarksService;
		   var BMDS  = RDF.GetDataSource("rdf:bookmarks");
		   var root = RDF.GetResource( "NC:BookmarksRoot" );
		   var BMSVC = BMDS.QueryInterface( kBMSVCIID );
		   var NameArc = RDF.GetResource( NC_NS+"Name" );

		   var curNameNode = BMDS.GetTarget( root, NameArc, true );
		   // necessary typecast
		   if( curNameNode instanceof kRDFLITIID );
		   var curFolder = { node: root, ancestors: [curNameNode.Value] };
		   var folderStack = new Array( curFolder );
		   var inOrExcludedFolders = bMHPrefs.getPref( "inOrExcludedFolders" ).split( "|" );
		   var excludePref = bMHPrefs.getPref( "excludeIndex" );
		   var exclude = ( excludePref == 0 ? true : false );
		   var enumerator, tempFolders, curNode; 
		   var curNodeAncestors, isChecked, isExcluded, i;

		   while( folderStack.length > 0 ) {
		       curFolder = folderStack.pop();
		       RDFC.Init( BMDS, curFolder.node );
		       enumerator = RDFC.GetElements();
		       tempFolders = new Array();
		       while( enumerator.hasMoreElements() ) {
			   curNode = enumerator.getNext();
			   curNameNode = BMDS.GetTarget( curNode, NameArc, true );
			   if( curNameNode instanceof kRDFLITIID ) { 
			       if( RDFCU.IsSeq( BMDS, curNode )) {
				   curNodeAncestors = curFolder.ancestors.concat( [curNameNode.Value] );
				   tempFolders.push( { node: curNode, ancestors: curNodeAncestors } );
			       }
			   }
		       }
		       isExcluded = bMHPrefs.isExcludedFolder( curFolder.ancestors.toString(),
			       excludePref, inOrExcludedFolders );
		       isChecked = ( exclude == isExcluded ? true : false );
		       bMHPrefsWindow.createFolder( curFolder.ancestors, isChecked ); 
		       // reverse to match order in bookmarks tree
		       folderStack = folderStack.concat( tempFolders.reverse() );
		   }
	       },

createFolder: function( ancestors, isChecked ) {  
		  // hbox for checkbox and indentation spacer
		  var folderBox = document.createElement( "hbox" );
		  this.inOrExcludedFoldersBox.appendChild( folderBox );
		  // spacer
		  var indentFolderSpacer = document.createElement( "spacer" );
		  indentFolderSpacer.setAttribute( "style", "width:" + (ancestors.length*35) + "px" );
		  folderBox.appendChild( indentFolderSpacer );
		  // checkbox
		  var folderCheckbox = document.createElement( "checkbox" );
		  folderCheckbox.setAttribute( "id", "fcb" + this.folderCheckboxArray.length );
		  folderCheckbox.setAttribute( "label", ancestors[ancestors.length-1] );
		  folderCheckbox.setAttribute( "checked", isChecked );
		  folderBox.appendChild( folderCheckbox );
		  // add checkbox to array
		  this.folderCheckboxArray.push( { node: folderCheckbox, value: ancestors } );
	      },
}


